/*
 * Project: MoleCuilder
 * Description: creates and alters molecular systems
 * Copyright (C)  2010-2012 University of Bonn. All rights reserved.
 * 
 *
 *   This file is part of MoleCuilder.
 *
 *    MoleCuilder is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    MoleCuilder is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with MoleCuilder.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * NotMoleculeByFormulaAction.cpp
 *
 *  Created on: May 12, 2010
 *      Author: heber
 */

// include config.h
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "CodePatterns/MemDebug.hpp"

#include "Descriptors/MoleculeFormulaDescriptor.hpp"
#include "Descriptors/MoleculeSelectionDescriptor.hpp"
#include "molecule.hpp"
#include "CodePatterns/Log.hpp"
#include "CodePatterns/Verbose.hpp"
#include "World.hpp"

#include <boost/foreach.hpp>
#include <iostream>
#include <string>

#include "NotMoleculeByFormulaAction.hpp"

using namespace MoleCuilder;

// and construct the stuff
#include "NotMoleculeByFormulaAction.def"
#include "Action_impl_pre.hpp"
/** =========== define the function ====================== */
ActionState::ptr SelectionNotMoleculeByFormulaAction::performCall() {
  LOG(1, "Unselecting molecules with chemical formula " << params.formula.get() << ":");
  std::vector<molecule *> matchingMolecules = World::getInstance().getAllMolecules(MoleculeByFormula(params.formula.get()));
  std::vector<molecule *> unselectedMolecules = World::getInstance().getAllMolecules(!MoleculesBySelection());
  World::getInstance().unselectAllMolecules(MoleculeByFormula(params.formula.get()));
  LOG(0, World::getInstance().countSelectedMolecules() << " molecules remain selected.");
  return ActionState::ptr(new SelectionNotMoleculeByFormulaState(unselectedMolecules,params));
}

ActionState::ptr SelectionNotMoleculeByFormulaAction::performUndo(ActionState::ptr _state) {
  SelectionNotMoleculeByFormulaState *state = assert_cast<SelectionNotMoleculeByFormulaState*>(_state.get());

  World::getInstance().selectAllMolecules(MoleculeByFormula(state->params.formula.get()));
  BOOST_FOREACH( molecule *mol, state->unselectedMolecules)
    World::getInstance().unselectMolecule(mol);

  return ActionState::ptr(_state);
}

ActionState::ptr SelectionNotMoleculeByFormulaAction::performRedo(ActionState::ptr _state){
  SelectionNotMoleculeByFormulaState *state = assert_cast<SelectionNotMoleculeByFormulaState*>(_state.get());

  World::getInstance().unselectAllMolecules(MoleculeByFormula(state->params.formula.get()));

  return ActionState::ptr(_state);
}

bool SelectionNotMoleculeByFormulaAction::canUndo() {
  return true;
}

bool SelectionNotMoleculeByFormulaAction::shouldUndo() {
  return true;
}
/** =========== end of function ====================== */
